<?php namespace TierPricingTable\Admin\ProductManagers;

use TierPricingTable\PriceManager;
use WC_Product;

/**
 * Class ProductManager
 *
 * @package TierPricingTable\Admin\Product
 */
class ProductManager extends ProductManagerAbstract {

	/**
	 * Register hooks
	 */
	protected function hooks() {

		// Tiered Pricing Product Tab
		add_filter( 'woocommerce_product_data_tabs', array( $this, 'registerTieredPricingProductTab' ), 99, 1 );
		add_action( 'woocommerce_product_data_panels', array( $this, 'renderTieredPricingTab' ) );
		add_action( 'woocommerce_process_product_meta', array( $this, 'saveTieredPricingTab' ) );

		// Simple Product
		add_action( 'woocommerce_product_options_pricing', array( $this, 'renderPriceRules' ) );

		// Saving
		add_action( 'woocommerce_process_product_meta', array( $this, 'updatePriceRules' ) );
		add_action( 'woocommerce_process_product_meta', array( $this, 'updatePriceRulesType' ) );
	}

	/**
	 * Add tiered pricing tab to woocommerce product tabs
	 *
	 * @param array $productTabs
	 *
	 * @return array
	 */
	public function registerTieredPricingProductTab( $productTabs ) {

		$productTabs['tiered-pricing-tab'] = array(
			'label'  => __( 'Tiered Pricing', 'tier-pricing-table' ),
			'target' => 'tiered-pricing-data',
			'class'  => array( 'show_if_simple', 'show_if_variable' )
		);

		return $productTabs;
	}

	/**
	 * Render content for the tiered pricing tab
	 */
	public function renderTieredPricingTab() {

		global $post;

		$min = PriceManager::getProductQtyMin( $post->ID, 'edit' );

		$desc = __( 'Set if you sell the product only from a specific quantity that is more than 1', 'tier-pricing-table' );

		$customAttrs = array(
			'min'  => 1,
			'step' => 1
		);

		?>
		<div id="tiered-pricing-data" class="panel woocommerce_options_panel">
			<?php
			/**
			 * Hook on the beginning of the tiered pricing tab
			 *
			 * @since 3.0.0
			 */
			do_action( 'tiered_pricing_table/admin/pricing_tab_begin', $post->ID );

			woocommerce_wp_text_input( array(
				'id'                => '_tiered_pricing_minimum',
				'wrapper_class'     => 'show_if_simple show_if_variable',
				'type'              => 'number',
				'custom_attributes' => $customAttrs,
				'value'             => $min,
				'label'             => __( 'Minimum quantity', 'tier-pricing-table' ),
				'description'       => $desc,
				'default'           => 1,
				'desc_tip'          => true
			) );
			?>

			<div class="hidden show_if_variable">
				<?php

				$type = PriceManager::getPricingType( $post->ID, 'fixed', 'edit' );

				$this->getContainer()->getFileManager()->includeTemplate( 'admin/add-price-rules.php', array(
					'price_rules_fixed'      => PriceManager::getFixedPriceRules( $post->ID, 'edit' ),
					'price_rules_percentage' => PriceManager::getPercentagePriceRules( $post->ID, 'edit' ),
					'type'                   => $type,
					'prefix'                 => 'variable',
				) );

				?>
			</div>

			<?php
			woocommerce_wp_select( array(
				'id'          => '_tiered_pricing_template',
				'value'       => self::getProductTemplate( $post->ID ),
				'options'     => array(
					'default' => __( 'Default', 'tier-pricing-table' ),
					'table'   => __( 'Table', 'tier-pricing-table' ),
					'blocks'  => __( 'Blocks', 'tier-pricing-table' ),
					'options' => __( 'Options', 'tier-pricing-table' ),
					'tooltip' => __( 'Tooltip', 'tier-pricing-table' ),
				),
				'label'       => __( 'Layout', 'tier-pricing-table' ),
				'description' => __( 'Specify a tiered pricing layout for the product. Leave the default to use the default layout from the settings', 'tier-pricing-table' ),
				'default'     => 'default',
				'desc_tip'    => true
			) );
			/**
			 * Before advanced product options
			 *
			 * @since 5.2.0
			 */
			do_action( 'tiered_pricing_table/admin/before_advance_product_options', $post->ID );
			?>

			<?php
			/**
			 * Display advanced product options if there are any
			 *
			 * @since 5.2.0
			 */
			if ( apply_filters( 'tiered_pricing_table/admin/has_advance_product_options', false, $post ) ) : 
				?>
				<div class="tiered_pricing_tab_product_advance_options">
					<div class="tiered_pricing_tab_product_advance_options__header">
						<h4>
							<?php esc_html_e( 'Advanced settings', 'tier-pricing-table' ); ?>
						</h4>
						<div>
							<span class="tiered_pricing_arrow_down">▼</span>
							<span class="tiered_pricing_arrow_up">▲</span>
						</div>
					</div>

					<div class="tiered_pricing_tab_product_advance_options__content">

						<?php
						/**
						 * Advanced product options
						 *
						 * @since 5.2.0
						 */
						do_action( 'tiered_pricing_table/admin/advance_product_options', $post->ID ); 
						?>
					</div>
				</div>
			<?php endif; ?>

			<?php
			/**
			 * Hook on the ending of the tiered pricing tab
			 *
			 * @since 3.0.0
			 */
			do_action( 'tiered_pricing_table/admin/pricing_tab_end', $post->ID );
			?>
		</div>
		<?php
	}

	/**
	 * Save tiered pricing tab data
	 *
	 * @param int $productId
	 */
	public function saveTieredPricingTab( $productId ) {

		if ( wp_verify_nonce( true, true ) ) {
			// as phpcs comments at Woo is not available, we have to do such a trash
			$woo = 'Woo, please add ignoring comments to your phpcs checker';
		}

		if ( isset( $_POST['_tiered_pricing_minimum'] ) ) {
			$min = intval( $_POST['_tiered_pricing_minimum'] );
			$min = $min > 0 ? $min : 1;

			PriceManager::updateProductQtyMin( $productId, $min );
		}
	}

	/**
	 * Update price quantity rules for simple product
	 *
	 * @param int $productId
	 */
	public function updatePriceRules( $productId ) {

		if ( wp_verify_nonce( true, true ) ) {
			// as phpcs comments at Woo is not available, we have to do such a trash
			$woo = 'Woo, please add ignoring comments to your phpcs checker';
		}

		$data = $_POST;

		$prefix = isset( $data['product-type'] ) && in_array( $data['product-type'],
			array( 'simple', 'variable' ) ) ? sanitize_text_field( $data['product-type'] ) : 'simple';

		$fixedAmounts = isset( $data[ 'tiered_price_fixed_quantity_' . $prefix ] ) ? (array) $data[ 'tiered_price_fixed_quantity_' . $prefix ] : array();
		$fixedPrices  = ! empty( $data[ 'tiered_price_fixed_price_' . $prefix ] ) ? (array) $data[ 'tiered_price_fixed_price_' . $prefix ] : array();

		PriceManager::updateFixedPriceRules( $fixedAmounts, $fixedPrices, $productId );

		$percentageAmounts = isset( $data[ 'tiered_price_percent_quantity_' . $prefix ] ) ? (array) $data[ 'tiered_price_percent_quantity_' . $prefix ] : array();
		$percentagePrices  = ! empty( $data[ 'tiered_price_percent_discount_' . $prefix ] ) ? (array) $data[ 'tiered_price_percent_discount_' . $prefix ] : array();
		PriceManager::updatePercentagePriceRules( $percentageAmounts, $percentagePrices, $productId );

		self::updateProductTemplate( $productId, isset( $data['_tiered_pricing_template'] ) ? $data['_tiered_pricing_template'] : 'default' );
	}

	/**
	 * Update product pricing type
	 *
	 * @param int $productId
	 */
	public function updatePriceRulesType( $productId ) {

		if ( wp_verify_nonce( true, true ) ) {
			// as phpcs comments at Woo is not available, we have to do such a trash
			$woo = 'Woo, please add ignoring comments to your phpcs checker';
		}

		$prefix = isset( $_POST['product-type'] ) && in_array( $_POST['product-type'],
			array( 'simple', 'variable' ) ) ? sanitize_text_field( $_POST['product-type'] ) : 'simple';

		if ( isset( $_POST[ 'tiered_price_rules_type_' . $prefix ] ) ) {
			PriceManager::updatePriceRulesType( $productId,
				sanitize_text_field( $_POST[ 'tiered_price_rules_type_' . $prefix ] ) );
		}

	}

	/**
	 * Render inputs for price rules on a simple product
	 *
	 * @global WC_Product $product_object
	 */
	public function renderPriceRules() {
		global $product_object;

		if ( $product_object instanceof WC_Product ) {
			$type = PriceManager::getPricingType( $product_object->get_id(), 'fixed', 'edit' );

			$this->getContainer()->getFileManager()->includeTemplate( 'admin/add-price-rules.php', array(
				'price_rules_fixed'      => PriceManager::getFixedPriceRules( $product_object->get_id(), 'edit' ),
				'price_rules_percentage' => PriceManager::getPercentagePriceRules( $product_object->get_id(), 'edit' ),
				'type'                   => $type,
				'prefix'                 => 'simple',
			) );
		}
	}

	public static function getProductTemplate( $productId ) {
		$template = get_post_meta( $productId, '_tiered_pricing_template', true );

		return in_array( $template, array(
			'default',
			'table',
			'blocks',
			'tooltip',
			'options'
		) ) ? $template : 'default';
	}

	public static function updateProductTemplate( $productId, $template ) {
		$template = in_array( $template, array(
			'default',
			'table',
			'blocks',
			'tooltip',
			'options'
		) ) ? $template : 'default';

		update_post_meta( $productId, '_tiered_pricing_template', $template );
	}
}
